function [fx0,fy0,fz0] = osf(fx,fy,fz)
%OSF retrieves current field points, sets new field points in OSLO, 
%    or sets one (or more) of the current fields already defined
%   NOTE: Please use OSSOP to set the object point for ray analyses
%
%   function [fx0,fy0,fz0] = osf(fx,fy,fz)
%
%   INPUT:  fx,fy,fz = sets new field data
%               -- two column input for fx is interpreted as [fx fy] data
%               -- fx only input (all integers) sets those current field points, removing
%               all others
%           NOTE: units should be absolute (i.e. in degrees or mm)
%
%   OUTPUT  fx0 fy0 fz0 = current field points defined in OSLO (prior to setting input)
%                  -- if only fx0 ouput is requested, 2 column data fx0 fy0 is given,
%                     or if fz0(:)~=0, then 3 column data is given
%           NOTE: output units are absolute (i.e. degrees / mm)
%
%   See also OSSOP, OSR, OSW, OSC
%

% get current field data in OSLO
if osnumf>0
    oscmd('sbr; rst prt fpt;');
    data = osssb(1:2:osnumf*2,1:3);
    fx0 = data(:,2); fy0 = data(:,1); fz0 = data(:,3);
else
    fx0 = []; fy0 = []; fz0 = [];
end

fldspec = osfap;
if fldspec==0 %object height
    scale = osreq('+obh;'); 
elseif fldspec==1 %object specified as angle
    scale = osreq('+ang/dr;'); %angle in degrees
elseif fldspec==2 
    scale = osreq('+gih;'); %guassian image height
end
fx0=fx0*scale; fy0=fy0*scale; fz0=fz0*scale;

if nargin>0  %set new field points
    oscmd('while(fptnbr>1) f 1 del;'); %erase current field points
    if nargin<2 
        if fx==int8(fx),  % get new field data from existing fields, if applicable
            fy = fy0(fx); fz = fz0(fx); fx = fx0(fx);
        else %set new field data from input
            c = size(fx,2);
            if c>2, fz = fx(:,3); else fz = zeros(length(fx(:,1)),1); end
            if c>1, fy = fx(:,2); else fy = zeros(length(fx(:,1)),1); end
            fx = fx(:,1);
        end
    elseif nargin<3 %fx fy input exists
        fz = zeros(length(fx),1);
    end
    % assign field points in OSLO
    for i=1:length(fx)
        oscmd(['f fptnbr ' num2str(fy(i)/scale,20) ' ' num2str(fx(i)/scale,20) ' ' num2str(fz(i)/scale,20) ';']);
    end
end

if nargout<2  %return multi-column data if only one output is requested
    if sum(abs(fz0(:)))<eps %only return x y data of z(:)==0
        fx0 = [fx0(:) fy0(:)];
    else
        fx0 = [fx0(:) fy0(:) fz0(:)];
    end
end

% Copyright  2004-2005 United States Government as represented by the Administrator of the 
% National Aeronautics and Space Administration.  No copyright is claimed in the United States 
% under Title 17, U.S. Code. All Other Rights Reserved.
% 
% Author: Joseph M. Howard, NASA
% Revision Date: 2007.08.22     
