function [rx0,ry0] = osr(rx,ry)
%OSR gets and sets ray definitions in OSLO,
%   function [rx0,ry0] = osr(x,y)
%
%   INPUT rx,ry = sets new ray data, removing previous data
%               -- two column input for rx is interpreted as [rx ry] data
%               -- integer rx only input (non-integer or multiple) sets 
%                  those current rays
%               -- negative integer sets a circular grid of rx density
%
%   OUTPUT rx0 ry0 = current rays defined in OSLO (prior to setting input)
%                  -- if only rx0 ouput is requested, 2 column data 
%                     [rx0 ry0] is returned
%
%   See also OSF, OSW, OSC
%

% get current ray data in OSLO
numr = osreq('raynbr-1;');
if numr>0
    oscmd('sbr; rst prt ray;');
    data = osssb(1:osnumr,1:2);
    rx0 = data(:,1); ry0 = data(:,2);
else
    rx0 = []; ry0 = [];
end

[fldspec,apspec] = osfap;
if apspec==0 %entrance beam radius
    scale = osreq('+ebr;'); 
elseif apspec==1 %object numerical aperture
    scale = osreq('+nao;');
end
rx0=rx0*scale; ry0=ry0*scale;

if nargin>0 
    oscmd('while(raynbr>1) r 1 del;'); %erase current rays
    % get new ray data from existing rays, if applicable
    if nargin<2 
        if size(rx,2)>1 % [rx ry] input for rx
            ry = rx(:,2); 
            rx = rx(:,1); 
        elseif rx==int8(rx), %integer input for rx
            ry = ry0(rx); rx = rx0(rx);
        end
    end
    % assign ray points in OSLO
    for i=1:length(rx)
        oscmd(['r raynbr ' num2str(ry(i)/scale,20) ' ' num2str(rx(i)/scale,20) ';']);
    end
end

if nargout<2  %return multi-column data if only one output is requested
    rx0 = [rx0(:) ry0(:)];
end

% Copyright  2004-2005 United States Government as represented by the Administrator of the 
% National Aeronautics and Space Administration.  No copyright is claimed in the United States 
% under Title 17, U.S. Code. All Other Rights Reserved.
% 
% Author: Joseph M. Howard, NASA
% Revision Date: 2007.08.22     
