function [dxy,dxyobj,drms,dopd,mask,opd0,xy0,xydxdy,dopl,opl0,drefrad,refrad0] = ossen(f,r,s,w,c,dx_da,ite_yn,ref_option)
% OSSEN calculates regid body sensitivities of image motion and wavefront
% function [dxy,dxyobj,drms,dopd,mask,opd0,xy0,xydxdy,dopl,opl0,drefrad,refrad0] 
%                   = ossen(f,r,s,w,c,dx_da,ite_yn,ref_option)
% INPUT:    f = desired field point for analysis, default = 1
%           r = nrd = pupil sampling, default = 64
%           s = surfaces that are perturbed (can be multiple)
%           w = wavelength number, default = 1
%           c = configuration number, default = 1
%           dx_da are motions applied (m/rad): defaults = 1 um urad
%           ite_yn = 0 if no iteration (i.e. compensators), = 1 to optimize
%           ref_option:  0 = min rms, 1 = best focus, 2 = ref ray,
%                        3 = stationary ref ray, i.e. no fit to tip/tilt
%                        4 = stationary ref ray, i.e. no fit to piston/tip/tilt
%                        (default = 0)
%
%   OUTPUT: sensitivities returned in m/m and m/rad = change/dx_da
%           xy = two column array of image motion of psf (based on ref sphere center)
%           xyobj = two column array of image motion of psf due to object shift
%           rms = rms value of opd from oswvf function (min rms opd)
%           opd, mask = opd sensitivities and mask data
%           xydxdy = x y dx dy raw data in 3 dimensions
%                      row = s, col = 6 DOF, 3rd dim = x,y,dx,dy
%           dopl, dopl0 = absolute OPL sensitivity of ref ray, and nominal
%           drefrad, drefrad0 = reference radius sensitivity and nominal
%
%   USAGE NOTE: to get LOS data with respect to object, perform xy/xyobj

%           ossen(nrd,s,f,w,z,dx_da);

% disp('Function is still being assembled... use another.'), return

if nargin<1, f = 1; end
if nargin<2, r = 32; end
if nargin<3, s = 1; end 
if nargin<4, w = 1; end 
if nargin<5, c = 1; end 
if nargin<6, dx_da = 1e-6; end %1 micron or micro-radian
if nargin<7, ite_yn = 0; end %compensation turned off
if nargin<8, ref_option = 0; end; %min rms

if ite_yn, ossave('junk.len'); end; % save file to "a.len" in current directory

nrd = r;
oscmd(['stp outp off; sdad ' int2str(nrd)]); %set nrd in OSLO

% get and set system data
units = osuni;
[fx, fy] = osf;
osw(w); 
osc(c);
dec = osdec(s,1); %get coord data in meters units

% Nominal data
[xy0,fxyz0,refrad0,opl0] = ossop(f); x0 = xy0(1); y0 = xy0(2);
[rmsval0,pk2val0,strehl0,dx0,dy0] = oswvf(0); %min rms oswvf(ref_option)
if nargout>3 || nargout<1, [opd0,mask0] = osgwf(nrd,ref_option); mask = mask0; end    

% Object sensitivity data for line-of-sight data
[fscale,fldspec] = osftyp;
if fldspec==1 %object angle
    [xy,fxyz,refrad,opl] = ossop([fx(f)+dx_da*180/pi fy(f)]/fscale); x = xy(1); y = xy(2);
    [rmsval,pk2val,strehl,dx,dy] = oswvf(0); %oswvf(ref_option);
    dxyobj(1,1) = (x+dx-x0-dx0); dxyobj(1,2) = (y+dy-y0-dy0);
    [xy,fxyz,refrad,opl] = ossop([fx(f) fy(f)+dx_da*180/pi]/fscale); x = xy(1); y = xy(2);
    [rmsval,pk2val,strehl,dx,dy] = oswvf(0); %oswvf(ref_option)
    dxyobj(2,1) = (x+dx-x0-dx0); dxyobj(2,2) = (y+dy-y0-dy0);
elseif fldspec==0 %object height
    [xy,fxyz,refrad,opl] = ossop([fx(f)+dx_da*1e3/units fy(f)]/fscale); x = xy(1); y = xy(2);
    [rmsval,pk2val,strehl,dx,dy] = oswvf(0); %oswvf(ref_option);
    dxyobj(1,1) = (x+dx-x0-dx0); dxyobj(1,2) = (y+dy-y0-dy0);
    [xy,fxyz,refrad,opl] = ossop([fx(f) fy(f)+dx_da*1e3/units]/fscale); x = xy(1); y = xy(2);
    [rmsval,pk2val,strehl,dx,dy] = oswvf(0); %oswvf(ref_option);
    dxyobj(2,1) = (x+dx-x0-dx0); dxyobj(2,2) = (y+dy-y0-dy0);
end

% Surface sensitivity data
xsen = zeros(length(s),6); ysen = xsen; rmssen = xsen;
for i=1:length(s) %loop on number of surfaces
    disp(['Working on surface ' int2str(i) ' of ' int2str(length(s))]);
    for j=1:6 %rigid body DOF
        if ite_yn, osopen('junk.len'); end; %open unperturbed "junk.len" file if iterating   
      % perturb lens
        osrbset(s(i),j,dec(i,j)+dx_da,1); %osrbshift(s(i),j,dx_da,1);
      % evaluate system
        if ite_yn, oscmd('ite 20',100); end
        [xy,fxyz,refrad(i,j),opl(i,j)] = ossop(f); x(i,j) = xy(1); y(i,j) = xy(2);
        [rmsval(i,j),pk2val,strehl,dx(i,j),dy(i,j)] = oswvf(0); %ref_option = 0
        if nargout>3 || nargout<1, 
            if ref_option<3, [opd(:,:,(i-1)*6+j),mask1] = osgwf(nrd,ref_option);
            else [opd(:,:,(i-1)*6+j),mask1] = osgwf(nrd,2, w, x(i,j)-x0, y(i,j)-y0, 0); 
            end
            mask = mask1&mask;
        end
      % reset surface
        osrbset(s(i),j,dec(i,j),1); %osrbshift(s(i),j,-dx_da,1);
    end
end

% calculate sensitivities
for i=1:length(s) %loop on number of surfaces
    for j=1:6 %rigid body DOF
        xsen(i,j) = x(i,j)+dx(i,j)-x0-dx0;
        ysen(i,j) = y(i,j)+dy(i,j)-y0-dy0;
        oplsen(i,j) = opl(i,j)-opl0;
        refradsen(i,j) = refrad(i,j)-refrad0;
        rmssen(i,j) = sqrt( abs(rmsval(i,j)*rmsval(i,j) - rmsval0*rmsval0) ); % Root-square difference
        if nargout>3 || nargout<1, opdsens(:,:,(i-1)*6+j) = opd(:,:,(i-1)*6+j)-opd0; end 
    end
end

% convert data to m/m or m/rad
xsen = xsen'; ysen = ysen'; %prepare for column formation
dxy = [xsen(:) ysen(:)]*1e-3*units/dx_da; %convert to m/m and m/rad
dxyobj = dxyobj*1e-3*units/dx_da; %convert to m/m and m/rad
drms = rmssen*1e-3*units/dx_da;  %convert to m/m and m/rad
if nargout>3 || nargout<1, dopd = opdsens/dx_da; end %opd is in m, convert to m/m and m/rad
xydxdy = cat(3,x,y,dx,dy)*1e-3*units; %convert to m/rad
dopl = oplsen*1e-3*units/dx_da;  %convert to m/m and m/rad
drefrad = refrad*1e-3*units/dx_da;  %convert to m/m and m/rad
xy0 = [x0 y0];

% add remove all piston data from ref_option==3, or add it to OPD if requested (ref_option==4)
if nargout>3 || nargout<1,
    if ref_option==3,
%         disp('Removing Piston data from OPD sensitivities');
%         for i=1:length(s) %loop on number of surfaces
%             for j=1:6 %rigid body DOF
%                 temp = dopd(:,:,(i-1)*6+j); 
%                 dopd(:,:,(i-1)*6+j) = mask.*(temp-mean(temp(mask)));
%             end
%         end
    elseif ref_option==4,
        disp('Adding OPL data to OPD sensitivities');
        for i=1:length(s) %loop on number of surfaces
            for j=1:6 %rigid body DOF
                dopd(:,:,(i-1)*6+j) = mask.*(dopd(:,:,(i-1)*6+j)+dopl(i,j));
            end
        end
    end
end

if nargout<1
    figure, subplot(2,1,1), plot(dxy/dxyobj,'o'), hold on, title('XY Line of Sight Sensitivities');
    if ref_option>3, subplot(2,1,2), plot(dopl,'o'), title('Piston Sensitivities'); end
end

% Copyright  2004-2005 United States Government as represented by the Administrator of the 
% National Aeronautics and Space Administration.  No copyright is claimed in the United States 
% under Title 17, U.S. Code. All Other Rights Reserved.
% 
% Author: Joseph M. Howard, NASA
% Revision Date: 2007.08.22     

